# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.arc.arc_adapter import ARCAdapter, DATASET_ID, SUBSET_LIST
from evalscope.benchmarks.arc.arc_adapter import ARCAdapter as DataAdapterClass
from evalscope.models.model_adapter import MultiChoiceModelAdapter as ModelAdapterClass    # noqa
