# Copyright (c) Alibaba, Inc. and its affiliates.


from evalscope.benchmarks.bbh.bbh_adapter import DATASET_ID, SUBSET_LIST
from evalscope.benchmarks.bbh.bbh_adapter import BBHAdapter as DataAdapterClass
from evalscope.models.model_adapter import ChatGenerationModelAdapter as ModelAdapterClass    # noqa
