# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.cmmlu.cmmlu_adapter import DATASET_ID, SUBJECT_MAPPING, SUBSET_LIST, CMMLUAdapter
from evalscope.benchmarks.cmmlu.cmmlu_adapter import CMMLUAdapter as DataAdapterClass
from evalscope.models.model_adapter import MultiChoiceModelAdapter as ModelAdapterClass    # noqa