# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.race.race_adapter import DATASET_ID, SUBJECT_MAPPING, SUBSET_LIST, RACEAdapter
from evalscope.benchmarks.race.race_adapter import RACEAdapter as DataAdapterClass
from evalscope.models.model_adapter import MultiChoiceModelAdapter as ModelAdapterClass    # noqa
