from evalscope.perf.plugin.datasets.custom import CustomDatasetPlugin
from evalscope.perf.plugin.datasets.flickr8k import FlickrDatasetPlugin
from evalscope.perf.plugin.datasets.line_by_line import LineByLineDatasetPlugin
from evalscope.perf.plugin.datasets.longalpaca import LongAlpacaDatasetPlugin
from evalscope.perf.plugin.datasets.openqa import OpenqaDatasetPlugin
from evalscope.perf.plugin.datasets.speed_benchmark import SpeedBenchmarkDatasetPlugin, SpeedBenchmarkLongDatasetPlugin
