# Copyright (c) Alibaba, Inc. and its affiliates.

from evalscope.benchmarks.trivia_qa.trivia_qa_adapter import TriviaQaAdapter, DATASET_ID, SUBSET_LIST
from evalscope.benchmarks.trivia_qa.trivia_qa_adapter import TriviaQaAdapter as DataAdapterClass
from evalscope.models.model_adapter import ChatGenerationModelAdapter as ModelAdapterClass    # noqa
